package j3d.examples.checkers;

// WrapCheckers3D.java
// Andrew Davison, January 2003, dandrew@ratree.psu.ac.th

/* Illustrates how to create a simple world. The checkboard
   floor is created by CheckerFloor. The background, lighting,
   key controls, and initial user positioning is done here.

   Most of this will stay the same from one example to another.

   floatingSphere() shows how to create a coloured, shiny object
   which is affected by the lighting in the world.

   The scene graph display code is in WrapCheckers3D() and 
   createSceneGraph().
*/


import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;

import javax.media.j3d.*;
import javax.swing.*;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import java.awt.*;

import j2d.ImageUtils;
import j2d.ShortImageBean;


public class CheckerPanel extends JPanel {
    private static final int PWIDTH = 512;
    // size of panel
    private static final int PHEIGHT = 512;

    private static final int BOUNDSIZE = 100;
    // larger than world

    private static final Point3d USERPOSN = new Point3d(0, 5, 20);
    // initial user position

    private SimpleUniverse su;
    private BranchGroup sceneBG;
    private BoundingSphere bounds;

    public CheckerPanel() {
        setLayout(new BorderLayout());
        setOpaque(false);
        setPreferredSize(new Dimension(PWIDTH, PHEIGHT));

        GraphicsConfiguration config =
                SimpleUniverse.getPreferredConfiguration();
        Canvas3D canvas3D = new Canvas3D(config);
        add(BorderLayout.CENTER, canvas3D);
        canvas3D.setFocusable(true);
        canvas3D.requestFocus();

        su = new SimpleUniverse(canvas3D);


        createSceneGraph();
        initUserPosition();
        // set user's viewpoint
        orbitControls(canvas3D);
        // controls for moving the viewpoint

        su.addBranchGraph(sceneBG);

        // j3dTree.updateNodes( su );    // build the tree display window

    } // end of WrapCheckers3D()


    private void createSceneGraph() {
        sceneBG = new BranchGroup();
        bounds = new BoundingSphere(new Point3d(0, 0, 0), BOUNDSIZE);

        lightScene();         // add the lights
        addBackground();      // add the sky
        sceneBG.addChild(new CheckerFloor().getBG());  // add the floor

        floatingSphere();     // add the floating sphere

        sceneBG.compile();   // fix the scene
    } // end of createSceneGraph()


    private void lightScene()
            /* One ambient light, 2 directional lights */ {
        Color3f white = new Color3f(1.0f, 1.0f, 1.0f);

        // Set up the ambient light
        AmbientLight ambientLightNode = new AmbientLight(white);
        ambientLightNode.setInfluencingBounds(bounds);
        sceneBG.addChild(ambientLightNode);

        // Set up the directional lights
        Vector3f light1Direction = new Vector3f(-1.0f, -1.0f, -1.0f);
        // left, down, backwards
        Vector3f light2Direction = new Vector3f(1.0f, -1.0f, 1.0f);
        // right, down, forwards

        DirectionalLight light1 =
                new DirectionalLight(white, light1Direction);
        light1.setInfluencingBounds(bounds);
        sceneBG.addChild(light1);

        DirectionalLight light2 =
                new DirectionalLight(white, light2Direction);
        light2.setInfluencingBounds(bounds);
        sceneBG.addChild(light2);
    }  // end of lightScene()


    private void addBackground() {
        Background back = new Background();
        back.setApplicationBounds(bounds);
        back.setColor(0.17f, 0.65f, 0.92f);    // sky colour
        sceneBG.addChild(back);
    }  // end of addBackground()


    private void orbitControls(Canvas3D c) {
        OrbitBehavior orbit =
                new OrbitBehavior(c, OrbitBehavior.REVERSE_ALL);
        orbit.setSchedulingBounds(bounds);

        ViewingPlatform vp = su.getViewingPlatform();
        vp.setViewPlatformBehavior(orbit);
    }


    private void initUserPosition()
            // Set the user's initial viewpoint using lookAt()
    {
        ViewingPlatform vp = su.getViewingPlatform();
        TransformGroup steerTG = vp.getViewPlatformTransform();

        Transform3D t3d = new Transform3D();
        steerTG.getTransform(t3d);

        // args are: viewer posn, where looking, up direction
        t3d.lookAt(USERPOSN, new Point3d(0, 0, 0), new Vector3d(0, 1, 0));
        t3d.invert();

        steerTG.setTransform(t3d);
    }  // end of initUserPosition()


    // ---------------------- floating sphere -----------------


    private void floatingSphere() {
        Image img = ImageUtils.getImage();
        ShortImageBean sib = new ShortImageBean(img);
        // use the ColorHash to create a list of unique colors
        // from an image.
        addColorSphere(new Color3f(Color.MAGENTA));
        addColorSphere(new Color3f(Color.RED));


    }  // end of floatingSphere()

    private void addColorSphere(Color3f c) {
        float f[] = new float[3];
        c.get(f);
        Vector3f location = new Vector3f(scale(f[0]),scale(f[1]),scale(f[2]));
        addSphere(location, c);
    }

    public static float scale(double x){
        return (float)(x * 10 - 10);
    }

    private void addSphere(Vector3f location, Color3f c) {
        // position the sphere
        Transform3D t3d = new Transform3D();

        t3d.set(location);
        TransformGroup tg = new TransformGroup(t3d);

        tg.addChild(getShape3D(getAppearance(c)));   // set its radius and appearance

        sceneBG.addChild(tg);
    }

    private Sphere getShape3D(Appearance a) {
        return new Sphere(1.0f, a);
    }

    private Appearance getAppearance(Color3f c) {

        Material a = getColoredMaterial(c);

        // sets ambient, emissive, diffuse, specular, shininess
        a.setLightingEnable(true);

        Appearance blueApp = new Appearance();
        blueApp.setMaterial(a);
        return blueApp;
    }

    private Material getColoredMaterial(Color3f c) {
        // Create the blue appearance node
        Color3f black = new Color3f(0.0f, 0.0f, 0.0f);

        Color3f specular = new Color3f(0.9f, 0.9f, 0.9f);

        Material blueMat = new Material(c, c, c, c, 25.0f);
        return blueMat;
    }

} // end of WrapCheckers3D class